#ifndef OSA_IOCTL_H
#define OSA_IOCTL_H

#include "../Api/OsaDef.h"


/*******************************************************************************
  Every OSA Ioctl struct must have one of these basic structs at the beginning.
  If not used they can be zeroed out but the space must be
  there.
*******************************************************************************/
typedef struct OsaIoctlInputS {
  OsaVoidP  pUserBuffer;
  OsaUint32 cbInUserBuffer;
  OsaUint32 cbUserBufferCapacity;
} OsaIoctlInputS, *OsaIoctlInputSP;


typedef struct OsaIoctlOutputS {
  OsaUint32 cbReturnedInUserBuffer;
} OsaIoctlOutputS, *OsaIoctlOutputSP;


/*******************************************************************************
*******************************************************************************/

#ifndef OSA_DRIVER
// Since this file is intended to be included in user mode as well
// we need to include the correct user mode system headers for the I/O control code handling
#if defined(WIN32)
#include <windows.h>
#include <winioctl.h>
#endif

#endif

#define OSA_DEVICE_NAME_MAX 48

#if OSA_WIN32 || defined(WIN32)
// Microsoft allows function code 2048 to 4096 for custom drivers
#define OSA_CTL_CODE(nFunctionCode) CTL_CODE(FILE_DEVICE_UNKNOWN, 2058 + nFunctionCode, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define OSA_IOCTL_DBG1 CTL_CODE(FILE_DEVICE_UNKNOWN, 2048 + 0, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define OSA_IOCTL_DBG2 CTL_CODE(FILE_DEVICE_UNKNOWN, 2048 + 1, METHOD_BUFFERED, FILE_ANY_ACCESS)

#else 
// Linux (user and kernel mode and all HW platforms)

#define OSA_IOC_MAGIC 'O'       // A magic value, not that important but we should choose something except 'T'

// I/O codes for the Os Extension Device
#define OSA_LINUX_OSEXT_IO_CREATE_EVENT  _IOWR(OSA_IOC_MAGIC, 0, 0)

// I/O codes for the User Event Device
#define OSA_LINUX_USREV_IO_SET_EVENT     _IOWR(OSA_IOC_MAGIC, 1, 0)

// Defines for making externally known User Event Device names
#define OSA_USER_EVENT_NAME_PREFIX  "osaevent"  // This name + suffix digits and \0 must never exceed OSA_DEVICE_NAME_MAX

struct OsaLinuxUserEventDevice;  // Forward declaration (opaque in user mode)

// Struct for returning a User Event Device name and 
typedef struct OsaLinuxIoOutCreateEvent {
  OsaIoctlOutputS sOsaIoctlBase;
  struct OsaLinuxUserEventDevice *pEventDevice;
  char szProcEntryName[OSA_DEVICE_NAME_MAX + 1];
} OsaLinuxIoOutCreateEvent;


#define OSA_CTL_CODE(nFunctionCode)  _IOWR(OSA_IOC_MAGIC, nFunctionCode, 0)    // We pass a size of 0 (which we use to indicate a driver io code, other values indicate a code for OSA

// Linux's ioctl() only allows a buffer and no extra size arguments etc. like windows DeviceIoControl
// so we need to supply a header at the beginning of that buffer. 
typedef struct OsaLinuxIoCtlHdr {

  void *pInBuf;
  OsaUint32 cbInBufSize;

  void *pOutBuf;
  OsaUint32 cbOutBufSize;

  OsaUint32 cbReturned;

} OsaLinuxIoCtlHdr;

#endif



#endif
